options = {
              'background_color':'#002B36',
              'text_color':'#839496',
              'status_text_color':'#002B36',
              'status_background_color':'#657b83',
              'text_highlight_color':'#657b83',
              'line_num_highlight_color':'#073642',
              'line_num_text_color':'#839496',
              'cursor_highlight_color':'#073642',
              'cursor_color':'#7C6B69',
              'function_name_color':'#b58900',
              'keyword_color':'#268bd2',
              'string_color':'#2aa198',
              'comment_color':'#586e75'
          }
